@@ -78,57 +78,113 @@ class MinterTx(object):
         for name, value in kwargs.items():
             setattr(self, name, value)
 
-    def sign(self, private_key, ms_address=None):
+    def generate_tx_rlp(self):
+        """
+        Create structure from instance and prepare rlp structure
+        Returns:
+            tx_struct (dict)
+        """
+        # Get structure populated with instance data
+        tx_struct = self._structure_from_instance()
+        # Remove signature data, it's not needed before getting Keccak
+        tx_struct.pop('signature_data')
+
+        # Encode tx data to RLP
+        tx_struct['data'] = rlp.encode(list(tx_struct['data'].values()))
+
+        return tx_struct
+
+    @staticmethod
+    def generate_signed_tx(tx_struct):
+        """
+        Generate signed tx hash from it's structure
+        Args:
+            tx_struct (dict): populated tx structure dict
+        Returns:
+            signed_tx hash (str)
+        """
+        tx_rlp = rlp.encode(list(tx_struct.values()))
+        return binascii.hexlify(tx_rlp).decode()
+
+    def generate_signature(self, private_key):
+        """
+        Create signature for transaction
+        Args:
+            private_key (str): private key to sign with
+        Returns:
+            hex_signature (str)
+        """
+        # Get structure populated with instance data and rlp encoded
+        tx_struct = self.generate_tx_rlp()
+
+        # Create keccak hash
+        tx_rlp = rlp.encode(list(tx_struct.values()))
+        keccak = MinterHelper.keccak_hash(tx_rlp)
+
+        # Create signature
+        signature = ECDSA.sign(keccak, private_key)
+        signature = binascii.hexlify(rlp.encode(signature)).decode()
+
+        return signature
+
+    def sign(self, private_key=None, signature=None, ms_address=None):
         """
         Sign transaction.
         This method can be called only from instances of inherited classes.
         Args:
-            private_key (string|list[string]): private key
+            private_key (string|list[string]): private key to sign transaction with
+            signature (string|list[string]): signature to sign transaction with
             ms_address (string): Multi signature address to sign tx by
-        Return:
-            string
         """
+        # Check arguments validity
+        if not private_key and not signature:
+            raise Exception('Please, provide either `private_key(s)` or `signature(s)`')
+        if not ms_address and private_key and type(private_key) is not str:
+            raise Exception('Please, provide a single `private_key` or set `ms_address` argument for multisig tx')
+        if not ms_address and signature and type(signature) is not str:
+            raise Exception('Please, provide a single `signature` or set `ms_address` argument for multisig tx')
 
         # Set tx signature type
         self.signature_type = self.SIGNATURE_SINGLE_TYPE
-
-        if not ms_address and type(private_key) is not str:
-            raise Exception('Please, provide a single `private_key` or set `ms_address` argument for multisig tx')
-
         if ms_address:
             if type(private_key) is str:
                 private_key = [private_key]
+            if type(signature) is str:
+                signature = [signature]
             self.signature_type = self.SIGNATURE_MULTI_TYPE
 
-        # Get structure populated with instance data
-        tx = self._structure_from_instance()
-        # Remove signature data, it's not needed before getting Keccak
-        tx.pop('signature_data')
-
-        # Encode tx data to RLP
-        tx['data'] = rlp.encode(list(tx['data'].values()))
-
-        # Encode all tx to RLP and create Keccak hash
-        tx_rlp = rlp.encode(list(tx.values()))
-        _keccak = MinterHelper.keccak_hash(tx_rlp)
+        # Get populated and rlp encoded tx structure
+        tx_struct = self.generate_tx_rlp()
 
         # Signature data
         if self.signature_type == self.SIGNATURE_SINGLE_TYPE:
-            signature = ECDSA.sign(_keccak, private_key)
+            # Only one of private_key or signature can be provided for single signature type tx
+            if private_key and signature:
+                raise Exception('Please, provide one of `private_key` or `signature` for single signature type tx')
+
+            # Set signature_data
+            signature_data = self.generate_signature(private_key) if private_key else signature
+            signature_data = self.decode_signature(signature_data)
         else:
             # Add multisig address to signature
-            signature = [
+            signature_data = [
                 MinterHelper.hex2bin(MinterPrefix.remove_prefix(string=ms_address, prefix=MinterPrefix.ADDRESS)),
                 []
             ]
 
-            # Sign by each private key and add to total signature
-            for pk in private_key:
-                signature[1].append(ECDSA.sign(_keccak, pk))
-        tx['signature_data'] = rlp.encode(signature)
+            # Sign by each private key and add to total signature data
+            if private_key:
+                for pk in private_key:
+                    _signature = self.generate_signature(pk)
+                    signature_data[1].append(self.decode_signature(_signature))
 
-        tx_rlp = rlp.encode(list(tx.values()))
-        self.signed_tx = binascii.hexlify(tx_rlp).decode()
+            # Sign by each signature and add to total signature data
+            if signature:
+                for _signature in signature:
+                    signature_data[1].append(self.decode_signature(_signature))
+        tx_struct['signature_data'] = rlp.encode(signature_data)
+
+        self.signed_tx = self.generate_signed_tx(tx_struct)
 
     def get_hash(self):
         """
@@ -332,6 +388,74 @@ class MinterTx(object):
 
         return MinterWallet.get_address_from_public_key(public_key)
 
+    @classmethod
+    def add_signature(cls, signed_tx, private_key):
+        """
+        Add signature to already signed tx. Method is available only for multisig txs
+        Args:
+            signed_tx (str): signed tx
+            private_key (str): private key
+        Returns:
+            tx object
+        """
+
+        # Create tx instance from raw tx
+        tx = cls.from_raw(signed_tx)
+
+        # Check tx to be multi signature type
+        if tx.signature_type != cls.SIGNATURE_MULTI_TYPE:
+            raise Exception('Signature can be added only to tx with multi signature type')
+
+        # Convert signature data from verbose dict to needed list of signatures
+        signature_data = [
+            MinterHelper.hex2bin(
+                MinterPrefix.remove_prefix(string=tx.signature_data['from_mx'], prefix=MinterPrefix.ADDRESS)
+            ),
+            []
+        ]
+        for item in tx.signature_data['signatures']:
+            # Create raw signature (values as integers)
+            raw = [item['v']]
+            raw.append(int(item['r'], 16))
+            raw.append(int(item['s'], 16))
+            # Append raw signature to total signature data
+            signature_data[1].append(raw)
+
+        # Get tx populated structure and keccak hash from this structure
+        tx_struct = tx.generate_tx_rlp()
+
+        # Create new signature and update signatures list and signatures attribute of tx
+        signature = cls.decode_signature(signature=tx.generate_signature(private_key))
+        signature_data[1].append(signature)
+        tx.signature_data['signatures'].append({
+            'v': signature[0],
+            'r': format(signature[1], 'x'),
+            's': format(signature[2], 'x')
+        })
+
+        # Update resulting struct signature data
+        tx_struct['signature_data'] = rlp.encode(signature_data)
+
+        # Generate new signed tx and update tx object attribute
+        tx.signed_tx = cls.generate_signed_tx(tx_struct)
+
+        return tx
+
+    @classmethod
+    def decode_signature(cls, signature):
+        """
+        Decode hexed signature to raw signature (list of integers)
+        Args:
+            signature (str): hexed signature
+        Returns:
+            signature (list[int])
+        """
+        signature = binascii.unhexlify(signature)
+        signature = rlp.decode(signature)
+        signature = [int(binascii.hexlify(item), 16) for item in signature]
+
+        return signature
+
     @classmethod
     def _data_from_raw(cls, raw_data):
         """
